<?php

/**
 * Une API pur erdv
 *
 * API à utiliser :
 *     ```
 *     include_spip('inc/erdv');
 *     ```
 * @plugin     Erdv
 * @copyright  2024
 * @author     Vincent CALLIES
 * @licence    GNU
 * @package    SPIP\Erdv\inc
 */

/**
 * Test de sécurité pour vérifier que SPIP est chargé 
 * et donc que les fichiers ne sont pas appelés en dehors de l'usage de SPIP
 */

if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * A partir d'une date, avoir les date de tous les jours de la semaine concernée
 *
 * @param int $time
 * @return array
 */
function semaine_en_cours(int $time) : array {
	// quel jour sommes-nous ? N donne 1 pour Lundi à 7 pour Dimanche
	$dj = date('N', $time);
	// avoir le tableau des jours de la semaine
	$i = 1; $semaine = [];
	while ($i <= 7) {
		if ($dj < $i) {
			$spectre = date('Y-m-d' , strtotime(' +' . ($i - $dj) . ' days', $time) ) . ' 00:00:00';
		} elseif ($dj > $i) {
			$spectre = date('Y-m-d' , strtotime( ($i - $dj) . ' days', $time) ) . ' 00:00:00';
		} elseif ($dj == $i) {
			$spectre = date('Y-m-d' , $time) . ' 00:00:00';
		}
		$semaine[] = $spectre;
		$i++; 
	}
	return $semaine;
}

/**
 * Consolider les rendez-vous sur une journée
 *
 * Un tétris à l'envers : on tente de placer les rendez-vous
 * sur la première ligne, et si ce n'est pas possible,
 * on descend.
 *
 * @param array $semaine
 * @param int $id_ecalendrier
 * @return array
 */
function tetris_journees(array $semaine, int $id_ecalendrier) : array {
	$lignes = []; $elements = [];
	$numero_de_la_semaine = date('W',strtotime($semaine[0]));
	if ($liens = sql_allfetsel('*', 'spip_erdvs', [
		'id_ecalendrier=' . intval($id_ecalendrier),
		sql_in('statut', ['publie']),
		'toute_la_journee=' . sql_quote('oui'),
		'DATE(date_debut) <= ' . sql_quote(date('Y-m-d', strtotime($semaine[6]))),
		'DATE(date_fin) >= ' . sql_quote(date('Y-m-d', strtotime($semaine[0]))),
			],
			'',
			'date_debut ASC')
		){
		$i = 0;
		foreach ($liens as $l) {
			$l['j'] = date('N',strtotime($l['date_debut'])) - 1;
			$l['k'] = date('N',strtotime($l['date_fin'])) - 1;
			// est-ce que data_debut est d'une semaine précédente ?
			if (date('W',strtotime($l['date_debut'])) < $numero_de_la_semaine){
				// le rendez-vous débute alors lundi
				$l['j'] = 0;
			}
			// est-ce que data_fin est d'une semaine suivante ?
			if (date('W',strtotime($l['date_fin'])) > $numero_de_la_semaine){
				// le rendez-vous finit alors dimanche
				$l['k'] = 6;
			}
			// Si le rendez-vous est récurrent est-ce la bonne journée ?
			if ($affiche = analyse_recurrence($l, $semaine)){ 
				// définir sur combien de jours/colonnes
				$l['colspan'] = ($l['k'] +1 ) - $l['j'];
				$elements[$i][$l['j']] = [
					'colspan' => $l['colspan'],
					'id_erdv' => $l['id_erdv'],
					'titre' => $l['titre'],
					'class' => $l['class'],
				];
				// indiquer que l'espace suivant est occupé si l'on est sur plusieurs lignes
				if ($l['colspan'] > 1){
					for($j = 1; $j < $l['colspan']; $j++){
						$elements[$i][$l['j'] + $j] = 'occupe';
					}
				}
				$i++;
			}
		}
		usort($elements, function($a, $b) {
			$aa = array_column($a,'colspan');
			$bb =  array_column($b,'colspan');
			if ($aa[0] == $bb[0]) {
				return 0;
			}
			return ($aa[0] < $bb[0]) ? 1 : -1;
		});
		// regarder ligne par ligne les $e éléments que l'on peut emboiter
		$e = count($elements);
		$i = 0; $n = 0;
		while ($n < $e) {
			// placer la première ligne
			$kf = array_key_first($elements);
			$lignes[$i] = $elements[$kf];
			unset($elements[$kf]);
			// est-ce qu'il y a de la place libre ?
			$combinaison=[];
			if (count($lignes[$i]) < 6){
				// La ligne $i a de la place libre !
				for ($c = 0; $c < 7; $c++) {
					if ( !isset($lignes[$i][$c]) ){
						// Le jour $c est libre
						$combinaison[]=$c;
					}
				}
				// on recherche l'élément compatible avec la combinaison
				$d = 0;
				foreach ($elements as $key => $values) {
					$occupe = false;
					foreach ($values as $jour => $infos) {
						// l'élément débute-t-il un jour de la combinaison ?
						// et un élément n'y a-t-il pas déjà été placé
						if ( 
							in_array($jour, $combinaison)
							and is_array($values[$jour])
							and !isset($lignes[$i][$jour])
							and $values[$jour]['colspan'] <= count($combinaison)
						) {
							// on a un élément gagnant : on le place
							//$lignes[$i] = array_merge($lignes[$i],$elements[$key]);
							$lignes[$i][$jour] = $infos;
							// on retire l'élément de la liste
							unset($elements[$key]);
							// on décompte
							$n++;
							// les autres colonnes de la ligne seront en occupe
							if ($infos['colspan'] > 1){
								for($add = 1; $add < $infos['colspan']; $add++){
									$lignes[$i][$jour + $add] = 'occupe';
								}
							}
						}
					}
				}
			}
			$n++;
			$i++;
		}
	}
	return $lignes;
}
/**
 * Consolider les rendez-vous par heure
 *
 * En colonne
 * to do : en ligne aussi
 *
 * @param array $semaine
 * @param int $id_ecalendrier
 * @return array
 */
function tetris_heures(array $semaine, int $id_ecalendrier) : array {
	$lignes = [];
	$numero_de_la_semaine = date('W',strtotime($semaine[0]));
	if ($liens = sql_allfetsel('*', 'spip_erdvs', [
				'id_ecalendrier=' . intval($id_ecalendrier),
				sql_in('statut', ['publie']),
				'toute_la_journee=' . sql_quote('non'),
				'DATE(date_debut) <= ' . sql_quote(date('Y-m-d', strtotime($semaine[6]))),
				'DATE(date_fin) >= ' . sql_quote(date('Y-m-d', strtotime($semaine[0]))),
			],
			'',
			'DATE_FORMAT(date_debut, "%H") ASC, titre ASC')
		){
		$i = 0;
		foreach ($liens as $l) {
			$l['j'] = date('N',strtotime($l['date_debut'])) - 1;
			$l['k'] = date('N',strtotime($l['date_fin'])) - 1;
			// est-ce que data_debut est d'une semaine précédente ?
			if (date('W',strtotime($l['date_debut']))<$numero_de_la_semaine){
				// le rendez-vous débute alors lundi
				$l['j'] = 0;
			}
			// est-ce que data_fin est d'une semaine suivante ?
			if (date('W',strtotime($l['date_fin'])) > $numero_de_la_semaine){
				// le rendez-vous finit alors dimanche
				$l['k'] = 6;
			}
			// Si le rendez-vous récurrent est-ce la bonne journée ?
			if ($affiche = analyse_recurrence($l, $semaine)){ 
				// définir sur combien de jours/colonnes
				$l['colspan'] = ($l['k'] +1 ) - $l['j'];
				// est-ce que la colonne de la ligne est occupée ?
				while (isset($lignes[$i][$l['j']])){
					// si oui on descend
					$i++;
				}
				// s'il faut plusieurs colonnes, sont-elles occupées ?
				if ($l['colspan']>1){
					for ($verif = $l['j']; $verif < ($l['j'] + $l['colspan']); $verif++) {
						while (isset($lignes[$i][$verif])){
							// si oui on descend
							$i++;
						}
					}
				}
				// s'il faut plusieurs colonnes le jour doit être précisé avant l'heure
				$heure_debut = date('H:i', strtotime($l['date_debut']));
				$heure_fin = date('H:i', strtotime($l['date_fin']));
				if ($l['colspan']>1) {
					$j_debut = 'date_jour_' . ( $l['j'] + 2 ) . '_abbr';
					$k_fin = $l['k'] + 2;
					if ($k_fin > 7) $k_fin = 1;
					$k_fin = "date_jour_{$k_fin}_abbr";
					$heure_debut = _T($j_debut) . ' ' . $heure_debut;
					$heure_fin = _T($k_fin) . ' ' . $heure_fin;
				}
				$lignes[$i][$l['j']] = [
					'heure_debut' => $heure_debut,
					'heure_fin' => $heure_fin,
					'colspan' => $l['colspan'],
					'id_erdv' => $l['id_erdv'],
					'titre' => $l['titre'],
					'class' => $l['class'],
				];
				// indiquer que l'espace suivant est occupé si l'on est sur plusieurs lignes
				if ($l['colspan'] > 1){
					for($j = 1; $j < $l['colspan']; $j++){
						$lignes[$i][$l['j'] + $j] = 'occupe';
					}
				}
			}
			$i = 0;$j = 0;
		}
	}
	return $lignes;
}

function analyse_recurrence(array &$l, array $semaine) : bool {
	// est-ce un rendez-vous récurrent sur une journée ?
	if ($l['recurrence_jour'] > 0){
		// le rendez-vous est alors avec un début et une fin au jour récurrent
		$jr = $l['recurrence_jour'] -1;
		$l['j'] = $l['k'] = $jr;
		// est-ce un jour récurrent sur une semaine ? (ie le lundi de la première semaine.
		// Attention, la première semaine peut ne pas avoir de lundi.)
		if ($l['recurrence_semaine'] > 0 and $l['recurrence_semaine'] < 6){
			// est-ce la bonne semaine ?
			$sr = date("W",strtotime($semaine[$jr])) - date("W",strtotime(date("Y-m-01"))) + 1;
			if ($sr != $l['recurrence_semaine']){
				return FALSE;
			}
		}
		// est-ce le xème jour récurrent du mois ? (ie le premier lundi du mois)
		if ($l['recurrence_semaine'] > 5){
			// avoir le premier jour du mois
			$time = mktime(0, 0, 0, date('n',strtotime($semaine[$jr])), 1, date('Y',strtotime($semaine[$jr])));
			// avoir le nom du jour recherché
			$nom = date('l', strtotime($semaine[$jr]));
			// on veut le premier jour (ie premier lundi du mois)
			$jour[6] = strtotime($nom, $time);
			$jour[7] = strtotime("next $nom", $jour[6]);
			$jour[8] = strtotime("next $nom", $jour[7]);
			$jour[9] = strtotime("next $nom", $jour[8]);
			// est le bon mois et jour ?
			if (
				date('n',strtotime($jour[$l['recurrence_semaine']])) !=  date('n',strtotime($semaine[$jr]))
				and
				$jour[$l['recurrence_semaine']] != strtotime($semaine[$jr])
			){
				return FALSE;
			}
		}
	}
	return TRUE;
}